@@ -1,26 +1,50 @@
+# file brnn/brnn.R
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 or 3 of the License
+#  (at your option).
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 #A package for Bayesian Regularized Neural Networks
-#Paulino Perez Rodriguez
-#Madison, WI, Dec. 2011
+#Author: Paulino Perez Rodriguez
+#Madison, WI, Sep. 2012
+#Birmingaham, Alabama, Jan. 2013
 
 #WARNING: This is an experimental version
 
-#This function rescales data between -1 and 1
-#Note is not the same that 
-#mapminmax in matlab, the matlab function will obtain the maximum and minimum for each 
-#column and then it will do the calculation
-normalize=function(y)
+#Normalize a vector or matrix, the resulting all 
+#resulting elements should be between -1 and 1
+#z=2(x-base)/spread - 1
+#where base=min(x), spread=max(x)-base
+
+normalize=function(x,base,spread)
 {
-  my_min=min(y)
-  my_max=max(y)
-  if(is.vector(y)) {y=2*(y-my_min)/(my_max-my_min)-1}
-  if(is.matrix(y)){
-    for(i in 1:ncol(y))
-    {
-       z=2*(y[,i]-my_min)/(my_max-my_min)-1
-       y[,i]=z
-    }
-  }
-  return(y)
+	if(is.matrix(x))
+	{
+		return(sweep(sweep(x,2,base,"-"),2,2/spread,"*")-1)
+	}else{
+	   	return(2*(x-base)/spread-1)
+	}
+}
+
+#Go back to the original scale
+#x=base+0.5*spread*(z+1)
+un_normalize=function(z,base,spread)
+{
+	if(is.matrix(z))
+	{
+		return(sweep(sweep(z+1,2,0.5*spread,"*"),2,base,"+"))
+	}else{
+		return(base+0.5*spread*(z+1))
+	}	
 }
 
 ii=function(element=1,times=1)
@@ -47,35 +71,9 @@ sech=function(x)
   2*exp(x)/(exp(2*x)+1)
 }
 
-#This function will obtain a prediction for a BASIC network
-#X is the matrix with data
-#theta is a list with the network parameters
-#neurons is the number of neurons
-predictions.nn=function(X,theta,neurons)
-{
-  n=nrow(X)
-  yhat=rep(NA,n)
-  for(i in 1:n)
-  {
-    suma=0
-    x=X[i,]
-    for(k in 1:neurons)
-    {
-      lambda=theta[[k]]
-      weight=lambda[1]
-      bias=lambda[2]
-      lambda=lambda[-c(1,2)]
-      z=bias+sum(lambda*x)
-      suma=suma+weight*tansig(z)
-    }
-    yhat[i]=suma
-  }
-  return(yhat)  
-}
-
 
 #This function will obtain predicted values for a basic NN
-#It uses an omp version of the routone predictions.nn to do the work
+#It uses an omp version of the routine predictions.nn to do the work
 #This is an internal function
 predictions.nn.C=function(vecX,n,p,theta,neurons,cores=1)
 {
@@ -96,38 +94,6 @@ Ew=function(theta)
    sum((unlist(theta))^2)
 }
 
-#This function will calculate the Jocobian for the errors
-#jacobian=function(y,X,theta,neurons)
-#{
-#   n=length(y)
-#   p=ncol(X)
-#   npar=neurons*(1+1+p)
-#   J=matrix(NA,nrow=n,ncol=npar)
-#   #For every row of the jacobian matrix
-#   for(i in 1:n)
-#   {
-#     #For every neuron
-#     x=X[i,]
-#     for(k in 1:neurons)
-#     {
-#       lambda=theta[[k]]
-#       weight=lambda[1]
-#       bias=lambda[2]
-#       lambda=lambda[-c(1,2)]
-#       z=bias+sum(lambda*x)
-#       dtansig=(sech(z))^2
-#       J[i,(1+1+p)*(k-1)+1]=tansig(z)                      #Derivate with respect to weight
-#       J[i,(1+1+p)*(k-1)+2]=weight*dtansig                 #Derivate with respect to bias
-#       #Derivate with respect to the beta's.
-#       for(j in 1:length(x))
-#       {
-#          J[i,(1+1+p)*(k-1)+j+2]=weight*dtansig*x[j]
-#       }
-#     }
-#   }
-#   return(-J)
-#}
-
 jacobian=function(vecX,n,p,npar,theta,neurons,cores=1)
 {
       vectheta=as.vector(unlist(theta))
@@ -213,7 +179,96 @@ estimate.trace=function(A,tol=1E-6,samples=40,cores=1)
     ans
 }
 
+brnn=function(x,...) UseMethod("brnn")
+brnn_extended=function(x,...) UseMethod("brnn_extended")
+
+
+#Pretty simple formula interface for brnn
+#Code adapted from nnet R package
+brnn.formula=function(formula,data,contrasts=NULL,...)
+{
+    m=match.call(expand.dots = FALSE)
+    if(is.matrix(eval.parent(m$data))) m$data=as.data.frame(data)
+    m$... = m$contrasts = NULL
+    m[[1L]] = as.name("model.frame")
+    m = eval.parent(m)
+    Terms = attr(m, "terms")
+    x =model.matrix(Terms, m, contrasts)
+    cons = attr(x, "contrast")
+    xint = match("(Intercept)", colnames(x), nomatch=0L)
+    if(xint > 0L) x = x[, -xint, drop=FALSE] 
+
+    y = model.response(m)
+	
+    out=brnn.default(x,y,...)
+
+    out$terms = Terms
+    out$coefnames = colnames(x)
+    out$call = match.call()
+    #res$na.action = attr(m, "na.action")
+    out$contrasts = cons
+    out$xlevels = .getXlevels(Terms, m)
+
+    class(out)=c("brnn.formula","brnn")
+
+    return(out)	
+}
+
+#Pretty simple formula interface for brnn_extended
+#Code adapted from Formula and betareg packages
+brnn_extended.formula=function(formula,data,contrastsx=NULL,contrastsz=NULL,...)
+{
+        if(missing(data)) data = environment(formula)
+  	mf = match.call(expand.dots = FALSE)
+  	m = match(c("formula", "data"), names(mf), 0L)
+  	mf = mf[c(1L, m)]
+  	mf$drop.unused.levels = TRUE
+
+  	## formula
+  	formula = as.Formula(formula)
+	if(length(formula)[1L]>1L) stop("Multiresponse not allowed in this model\n");
+	if(length(formula)[2L]!=2L) stop("Two groups of predictors should be separated by | ");
+  	
+  	mf$formula = formula
+
+  	## evaluate model.frame
+  	mf[[1L]] = as.name("model.frame")
+  	mf = eval(mf, parent.frame())
+
+  	## extract terms, model matrix, response
+  	mt = terms(formula, data = data)
+  	mtx = terms(formula, data = data, rhs = 1L)
+  	mtz = delete.response(terms(formula, data = data, rhs = 2L))
+  	y = model.response(mf, "numeric")
+  	x = model.matrix(mtx, mf,contrastsx)
+        consx = attr(x, "contrast")
+  	z = model.matrix(mtz, mf,contrastsz)
+        consz = attr(z, "contrast")
+  	
+  	xint = match("(Intercept)", colnames(x), nomatch=0L)
+    	if(xint > 0L) x = x[, -xint, drop=FALSE] # Drop intecept
+    
+    	zint = match("(Intercept)", colnames(z), nomatch=0L)
+    	if(zint > 0L) z = z[, -zint, drop=FALSE] # Drop intecept
+  
+    	out=brnn_extended.default(x,y,z,...)
+    	out$call=match.call()
+	
+	out$mtx=mtx
+	out$mtz=mtz
+        out$contrastsx=consx
+        out$contrastsz=consz
+	out$xlevels = .getXlevels(mtx, mf)
+        out$zlevels = .getXlevels(mtz, mf)
+	out$call=match.call()
+
+	class(out)=c("brnn_extended.formula","brnn_extended")
+
+    	return(out)
+}
+
 #Function to do Bayesian Regularization for a network with a single layer and S neurons
+#normalize, logical if TRUE will rescale inputs and output in the interval [-1,1]
 #mu, strict pos scalar, default value 0.005
 #mu_dec,strict positive scalar, is the mu decrease ratio, default value 0.1
 #mu_inc, strict positive scalar, is the mu increase ratio, default value 10
@@ -225,20 +280,41 @@ estimate.trace=function(A,tol=1E-6,samples=40,cores=1)
 #tol, tolerance, argument for estimate.trace
 #samples number of Monte Carlo reps used for Monte Carlo estimates, argument for estimate.trace
 
-brnn=function(y,X,neurons,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10,
-              min_grad=1e-10,change=0.001,cores=1,verbose=TRUE,
-              Monte_Carlo=FALSE,tol=1E-6,samples=40)
+brnn.default=function(x,y,neurons=2,normalize=TRUE,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10,
+              min_grad=1e-10,change=0.001,cores=1,verbose=FALSE,
+              Monte_Carlo=FALSE,tol=1E-6,samples=40,...)
 {
    reason="UNKNOWN";
 
    #Checking that the imputs are ok
    if(!is.vector(y)) stop("y must be a vector\n")
-   if(!is.matrix(X)) stop("X must be a matrix\n")
-   vecX=as.vector(X)
+   if(!is.matrix(x)) stop("x must be a matrix\n")
+   
+   if(normalize)
+   {
+        x_base=apply(x,2,min)
+        x_spread=apply(x,2,max)-x_base 
+        x_normalized=normalize(x,base=x_base,spread=x_spread)
+   		
+   		y_base=min(y)
+   		y_spread=max(y) - y_base
+   		y_normalized=normalize(y,base=y_base,spread=y_spread)
+   		
+   }else{
+   		y_normalized=y
+   		y_base=NULL
+   		y_spread=NULL
+   		
+   		x_normalized=x
+   		x_base=NULL
+   		x_spread=NULL
+   }
+   
+   vecx=as.vector(x_normalized)
    
    #Initializing parameters for the net
-   p=ncol(X)
-   n=length(y)
+   p=ncol(x_normalized)
+   n=length(y_normalized)
    
    #neurons is the number of neurons
    #the first 1 corresponds to the weight for the tansig function, i.e. weight*tansig(.)
@@ -250,18 +326,14 @@ brnn=function(y,X,neurons,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10
    theta=initnw(neurons,p,n,npar)
  
    gamma=npar
-
-   #e=y-predictions.nn(X,theta,neurons)
-   e=y-predictions.nn.C(vecX,n,p,theta,neurons,cores)
+   
+   e=y_normalized-predictions.nn.C(vecx,n,p,theta,neurons,cores)
 
    Ed=sum(e^2)
    beta=(n - gamma)/(2*Ed); if(beta<0) beta=1;
    Ew=Ew(theta)
    alpha=gamma/(2*Ew)
 
-   #alpha=0.01
-   #beta=1
-
    epoch=1
    flag_gradient=TRUE
    flag_mu=TRUE
@@ -276,18 +348,15 @@ brnn=function(y,X,neurons,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10
    {
       if(verbose)
       {
-	cat("----------------------------------------------------------------------\n")
-	cat("Epoch=",epoch,"\n")
+		cat("----------------------------------------------------------------------\n")
+		cat("Epoch=",epoch,"\n")
       }
       
-      #J=jacobian(y,X,theta,neurons)
-      #parallel version
-
-      J=jacobian(vecX,n,p,npar,theta,neurons,cores)
+      J=jacobian(vecx,n,p,npar,theta,neurons,cores)
       
       H=crossprod(J)
-      #e=y-predictions.nn(X,theta,neurons)
-      e=y-predictions.nn.C(vecX,n,p,theta,neurons,cores)
+      
+      e=y_normalized-predictions.nn.C(vecx,n,p,theta,neurons,cores)
 
       #g=2*beta*t(J)%*%e+2*alpha*unlist(theta)
       g=2*as.vector((beta*t(e)%*%J+alpha*unlist(theta)))  #is it faster?
@@ -298,8 +367,8 @@ brnn=function(y,X,neurons,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10
       C=beta*Ed+alpha*Ew
       if(verbose)
       {
-	cat("C=",C,"\tEd=",Ed,"\tEw=",Ew,"\n")
-	cat("gradient=",mg,"\n")
+		cat("C=",C,"\tEd=",Ed,"\tEw=",Ew,"\n")
+		cat("gradient=",mg,"\n")
       }
        
       F_history[epoch]=C_new
@@ -316,34 +385,34 @@ brnn=function(y,X,neurons,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10
       flag_mu=mu<=mu_max
       while(flag_C & flag_mu)
       {
-	  tmp=as.vector(unlist(theta)-solve(2*beta*H+ii(2*alpha+mu,npar),g))
-	  theta_new=list()
-	  for(i in 1:neurons)
-	  {
-	      theta_new[[i]]=tmp[1:(2+p)]
-	      tmp=tmp[-c(1:(2+p))]
-	  }
-	  #e_new=y-predictions.nn(X,theta_new,neurons)
-          e_new=y-predictions.nn.C(vecX,n,p,theta_new,neurons,cores)
-	  Ed=sum(e_new^2)
-	  Ew=Ew(theta_new)
-	  C_new=beta*Ed+alpha*Ew
-	  if(verbose) 
-	  {
-	    cat("C_new=",C_new,"\tEd=",Ed,"\tEw=",Ew,"\n")
-	  }
-	  if(C_new<C)
-	  {
-	    mu=mu*mu_dec
-            if (mu < 1e-20) mu = 1e-20;
-	    flag_C=FALSE
-	  }else{
-	    mu=mu*mu_inc
-	  }
-	  if(verbose)
-	  {
-	    cat("mu=",mu,"\n")
-	  }
+	  	tmp=as.vector(unlist(theta)-solve(2*beta*H+ii(2*alpha+mu,npar),g))
+	  	theta_new=list()
+	  	for(i in 1:neurons)
+	  	{
+	      		theta_new[[i]]=tmp[1:(2+p)]
+	      		tmp=tmp[-c(1:(2+p))]
+	  	}
+	  
+      		e_new=y_normalized-predictions.nn.C(vecx,n,p,theta_new,neurons,cores)
+	  	Ed=sum(e_new^2)
+	  	Ew=Ew(theta_new)
+	  	C_new=beta*Ed+alpha*Ew
+	  	if(verbose) 
+	  	{
+	    		cat("C_new=",C_new,"\tEd=",Ed,"\tEw=",Ew,"\n")
+	  	}
+	  	if(C_new<C)
+	  	{
+	    		mu=mu*mu_dec
+            		if (mu < 1e-20) mu = 1e-20;
+	    		flag_C=FALSE
+	  	}else{
+	    		mu=mu*mu_inc
+	  	}
+	  	if(verbose)
+	  	{
+	    		cat("mu=",mu,"\n")
+	  	}
           flag_mu=mu<=mu_max
       }
       #Update all
@@ -374,31 +443,72 @@ brnn=function(y,X,neurons,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10
    {
      cat("gamma=",round(gamma,4),"\t","alpha=",round(alpha,4),"\t","beta=",round(beta,4),"\n")
    }
+
    #answer
-   return(list(theta=theta,alpha=alpha,beta=beta,gamma=gamma,Ed=Ed,Ew=Ew,F_history=F_history,reason=reason,epoch=epoch))
+   out=list(theta=theta,alpha=alpha,beta=beta,gamma=gamma,Ed=Ed,Ew=Ew,F_history=F_history,reason=reason,epoch=epoch, 
+            neurons=neurons,p=p,n=n,npar=npar,x_normalized=x_normalized,x_base=x_base,x_spread=x_spread,
+            y_base=y_base,y_spread=y_spread,y=y,
+            normalize=normalize)
+   out$call=match.call()
+   class(out)="brnn";
+   
+   #return the goodies
+   return(out)
 }
 
-brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10,
-                       min_grad=1e-10,change=0.001,cores=1,verbose=TRUE)
+brnn_extended.default=function(x,y,z,neurons1,neurons2,normalize=TRUE,epochs=1000,mu=0.005,mu_dec=0.1, mu_inc=10,mu_max=1e10,
+                       min_grad=1e-10,change=0.001,cores=1,verbose=FALSE,...)
 {
    reason="UNKNOWN";
 
    #Checking that the imputs are ok
    if(!is.vector(y)) stop("y must be a vector\n")
-   if(!is.matrix(X1)) stop("X1 must be a matrix\n") 
-   if(!is.matrix(X2)) stop("X2 must be a matrix\n")
-   vecX1=as.vector(X1)
-   vecX2=as.vector(X2)
+   if(!is.matrix(x)) stop("x must be a matrix\n") 
+   if(!is.matrix(z)) stop("z must be a matrix\n")
+   
+   
+   if(normalize)
+   {
+        x_base=apply(x,2,min)
+        x_spread=apply(x,2,max)-x_base 
+        x_normalized=normalize(x,base=x_base,spread=x_spread)
+        
+        z_base=apply(z,2,min)
+        z_spread=apply(z,2,max)-z_base 
+        z_normalized=normalize(z,base=z_base,spread=z_spread)
+   		
+   	y_base=min(y)
+   	y_spread=max(y) - y_base
+   	y_normalized=normalize(y,base=y_base,spread=y_spread)
+   		
+   }else{
+
+   	y_normalized=y
+   	y_base=NULL
+   	y_spread=NULL
+   		
+   	x_normalized=x
+   	x_base=NULL
+   	x_spread=NULL
+   		
+   	z_normalized=z
+   	z_base=NULL
+   	z_spread=NULL
+   }
+   
+   
+   vecx=as.vector(x_normalized)
+   vecz=as.vector(z_normalized)
    
    #Initializing parameters for the net
-   p=ncol(X1)
-   q=ncol(X2)
-   n=length(y)
+   p=ncol(x_normalized)
+   q=ncol(z_normalized)
+   n=length(y_normalized)
    
    #neurons is the number of neurons
    #the first 1 corresponds to the weight for the tansig function, i.e. weight*tansig(.)
    #the second 1 corresponds to the bias in tansig(.) function, .= bias + xi[1]*beta[1]+...+xi[p]*beta[p]
-   #p corresponds to the number of covariates
+   #p and q corresponds to the number of covariates
 
    npar1=neurons1*(1+1+p)
    npar2=neurons2*(1+1+q)
@@ -419,11 +529,7 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
    alpha=0.01
    delta=0.01
    beta=1
-   alpha_c=0.01
-   delta_c=0.01
-   w1=0.1
-   w2=0.1
-  
+   
    epoch=1
    flag_gradient=TRUE
    flag_mu=TRUE
@@ -437,41 +543,37 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
    {
       if(verbose)
       {
-	cat("----------------------------------------------------------------------\n")
-	cat("Epoch=",epoch,"\n")
+		cat("----------------------------------------------------------------------\n")
+		cat("Epoch=",epoch,"\n")
       }
      
-      #J1=jacobian(y,X1,theta1,neurons1)
-      #J2=jacobian(y,X2,theta2,neurons2)
-      #J=cbind(J1,J2,-predictions.nn(X1,theta1,neurons1),-predictions.nn(X2,theta2,neurons2))
-  
       #Parallel version  
-      J1=jacobian(vecX1,n,p,npar1,theta1,neurons1,cores)
-      J2=jacobian(vecX2,n,q,npar2,theta2,neurons2,cores)
-      p1=predictions.nn.C(vecX1,n,p,theta1,neurons1,cores)
-      p2=predictions.nn.C(vecX2,n,q,theta2,neurons2,cores)
-      J=cbind(J1,J2,-p1,-p2)
+      J1=jacobian(vecx,n,p,npar1,theta1,neurons1,cores)
+      J2=jacobian(vecz,n,q,npar2,theta2,neurons2,cores)
+      J=cbind(J1,J2)
 
     
       H=crossprod(J)
-
-      e=y-w1*p1-w2*p2
       
-      #g=2*beta*t(J)%*%e+2*c(alpha*unlist(theta1),delta*(unlist(theta2)),c(alpha_c*w1,delta_c*w2))
-      g=2*as.vector((beta*t(e)%*%J+c(alpha*unlist(theta1),delta*(unlist(theta2)),c(alpha_c*w1,delta_c*w2))))  #is it faster?
+      p1=predictions.nn.C(vecx,n,p,theta1,neurons1,cores)
+      p2=predictions.nn.C(vecz,n,q,theta2,neurons2,cores)
+
+      e=y_normalized-p1-p2
+      
+      #g=2*beta*t(J)%*%e+2*c(alpha*unlist(theta1),delta*(unlist(theta2)))
+      g=2*as.vector((beta*t(e)%*%J+c(alpha*unlist(theta1),delta*(unlist(theta2))))) #is it faster?
       
       mg=max(abs(g))
       flag_gradient=mg>min_grad
       Ed=sum(e^2)
       E1=Ew(theta1)
       E2=Ew(theta2)
-      E3=w1^2
-      E4=w2^2
-      C=beta*Ed+alpha*E1+delta*E2+alpha_c*E3+delta_c*E4
+      
+      C=beta*Ed+alpha*E1+delta*E2
       if(verbose)
       {
-	cat("C=",C,"\tEd=",Ed,"\tE1=",E1,"\tE2=",E2,"\tE3=",E3,"\tE4=",E4,"\n")
-	cat("gradient=",mg,"\n")
+		cat("C=",C,"\tEd=",Ed,"\tE1=",E1,"\tE2=",E2,"\n")
+		cat("gradient=",mg,"\n")
       }
 
       F_history[epoch]=C_new
@@ -481,7 +583,7 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
           if(max(abs(diff(F_history[(epoch-3):epoch])))<change) 
           {
               flag_change_F=FALSE;
-              reason=paste("Changes in F=beta*SCE + alpha * Ea + delta *Ed +alpha_c*c_a^2 + delta_c*c_d^2 in last 3 iterations less than",change,sep=" ");
+              reason=paste("Changes in F=beta*SCE + alpha * Ea + delta *Ed in last 3 iterations less than",change,sep=" ");
           }
       }
       
@@ -489,14 +591,15 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
       flag_mu=mu<=mu_max
       while(flag_C & flag_mu)
       {
-          Q=diag(c(rep(2*alpha+mu,npar1),rep(2*delta+mu,npar2),c(2*alpha_c+mu,2*delta_c+mu)))
-	  tmp=as.vector(c(unlist(theta1),unlist(theta2),c(w1,w2))-solve(2*beta*H+Q,g))
+          Q=diag(c(rep(2*alpha+mu,npar1),rep(2*delta+mu,npar2)))
+	  tmp=as.vector(c(unlist(theta1),unlist(theta2))-solve(2*beta*H+Q,g))
 
 	  theta_new1=list()
+
 	  for(i in 1:neurons1)
 	  {
-	      theta_new1[[i]]=tmp[1:(2+p)]
-	      tmp=tmp[-c(1:(2+p))]
+		theta_new1[[i]]=tmp[1:(2+p)]
+	      	tmp=tmp[-c(1:(2+p))]
 	  }
 
           theta_new2=list()
@@ -504,27 +607,24 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
           {
               theta_new2[[i]]=tmp[1:(2+p)]
 	      tmp=tmp[-c(1:(2+p))]
-          }
-          w1_new=tmp[1]
-          w2_new=tmp[2]	  
+          }  
+          
           
           #Paralell version
-          p1_new=predictions.nn.C(vecX1,n,p,theta_new1,neurons1,cores)
-          p2_new=predictions.nn.C(vecX2,n,q,theta_new2,neurons2,cores)
+          p1_new=predictions.nn.C(vecx,n,p,theta_new1,neurons1,cores)
+          p2_new=predictions.nn.C(vecz,n,q,theta_new2,neurons2,cores)
       
-          e_new=y-w1_new*p1_new-w2_new*p2_new
+          e_new=y_normalized-p1_new-p2_new
 
 	  Ed=sum(e_new^2)
 	  E1=Ew(theta_new1)
           E2=Ew(theta_new2)
-          E3=w1_new^2
-          E4=w2_new^2
 
-	  C_new=beta*Ed+alpha*E1+delta*E2+alpha_c*E3+delta_c*E4
+	  C_new=beta*Ed+alpha*E1+delta*E2
 
 	  if(verbose)
 	  {
-	    cat("C_new=",C_new,"\tEd=",Ed,"\tE1=",E1,"\tE2=",E2,"\tE3=",E3,"\tE4=",E4,"\n")
+		cat("C_new=",C_new,"\tEd=",Ed,"\tE1=",E1,"\tE2=",E2,"\n")
 	  }
 	  if(C_new<C)
 	  {
@@ -544,22 +644,17 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
      
       theta1=theta_new1
       theta2=theta_new2
-      w1=w1_new
-      w2=w2_new
+      
       epoch=epoch+1
-      Q1=diag(c(rep(2*alpha,npar1),rep(2*delta,npar2),c(2*alpha_c,2*delta_c)))
+      Q1=diag(c(rep(2*alpha,npar1),rep(2*delta,npar2)))
       d=diag(solve(2*beta*H+Q1))
       gamma1=npar1-2*alpha*sum(d[1:npar1])
       gamma2=npar2-2*delta*sum(d[(npar1+1):(npar1+npar2)])
-      gamma3=1-2*alpha_c*d[npar1+npar2+1]; if(gamma3<0) gamma3=0.1
-      gamma4=1-2*delta_c*d[npar1+npar2+2]; if(gamma4<0) gamma4=0.1
-
+    
       alpha=gamma1/(2*E1)
       delta=gamma2/(2*E2)
-      alpha_c=gamma3/(2*E3)
-      delta_c=gamma4/(2*E4)
       
-      beta=(n-gamma1-gamma2-gamma3-gamma4)/(2*Ed)
+      beta=(n-gamma1-gamma2)/(2*Ed)
       
       if(Ed<0.01)
       {
@@ -568,8 +663,8 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
       }
       if(verbose)
       {
-	cat("gamma_a=",round(gamma1,4),"\tgamma_delta=",round(gamma2,4),"\tgamma_alpha_c=",round(gamma3,4),"\tgamma_delta_c=",round(gamma4,4),"\n")
-	cat("alpha=",round(alpha,4),"\tdelta=",round(delta,4),"alpha_c=",round(alpha_c,4),"delta_c=",round(delta_c,4),"\tbeta=",round(beta,4),"\n")
+		cat("gamma_a=",round(gamma1,4),"\tgamma_delta=",round(gamma2,4),"\n")
+		cat("alpha=",round(alpha,4),"\tdelta=",round(delta,4),"\tbeta=",round(beta,4),"\n")
       }
    }
    if((epoch-1)==epochs) reason="Maximum number of epochs reached";
@@ -578,17 +673,28 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
    
    if(!verbose)
    {
-      cat("gamma_a=",round(gamma1,4),"\tgamma_delta=",round(gamma2,4),"\tgamma_alpha_c=",round(gamma3,4),"\tgamma_delta_c=",round(gamma4,4),"\n")
-      cat("alpha=",round(alpha,4),"\tdelta=",round(delta,4),"alpha_c=",round(alpha_c,4),"delta_c=",round(delta_c,4),"\tbeta=",round(beta,4),"\n")
+      cat("gamma_a=",round(gamma1,4),"\tgamma_delta=",round(gamma2,4),"\n")
+      cat("alpha=",round(alpha,4),"\tdelta=",round(delta,4),"\tbeta=",round(beta,4),"\n")
    }
-   return(list(theta1=theta1,theta2=theta2,c_a=w1,c_d=w2,alpha=alpha,beta=beta,
-               delta=delta,alpha_c=alpha_c,delta_c=delta_c,
-               E1=E1,E2=E2,reason=reason,epoch=epoch,F_history=F_history))
+   
+   out=list(theta1=theta1,theta2=theta2,alpha=alpha,beta=beta,
+               delta=delta,
+               E1=E1,E2=E2,reason=reason,epoch=epoch,F_history=F_history,
+               x_normalized=x_normalized,x_base=x_base,x_spread=x_spread,
+               z_normalized=z_normalized,z_base=x_base,z_spread=z_spread,
+               y_base=y_base,y_spread=y_spread,y=y,
+               neurons1=neurons1, neurons2=neurons2,
+               n=n,p=p,q=q,npar1=npar1,npar2=npar2, 
+               normalize=normalize)
+               
+   class(out)="brnn_extended";
+
+   return(out);
 }
 
-.First.lib <- function(lib, pkg) {
-  library.dynam("brnn", pkg, lib)
-}
+#.First.lib <- function(lib, pkg) {
+#  library.dynam("brnn", pkg, lib)
+#}
 
 ##################################################################################################
 .onAttach <- function(library, pkg)
@@ -598,7 +704,7 @@ brnn.extended=function(y,X1,X2,neurons1,neurons2,epochs=1000,mu=0.005,mu_dec=0.1
     stop("This package requires R 2.13.2 or later")
   assign(".brnn.home", file.path(library, pkg),
          pos=match("package:brnn", search()))
-  brnn.version <- "0.1 (2012-07-31)"
+  brnn.version <- "0.3 (2013-03-15)"
   assign(".brnn.version", brnn.version, pos=match("package:brnn", search()))
   if(interactive())
   {
